
class TextFormat:
    def without_acute(self, text: str) -> str:
        acutes = {
            'á': 'a',
            'é': 'e',
            'í': 'i',
            'ó': 'o',
            'ú': 'u',
            'Á': 'A',
            'É': 'E',
            'Í': 'I',
            'Ó': 'O',
            'Ú': 'U'
        }
        for acute in acutes:
            text = text.replace(acute, acutes[acute])
        return text
    
    def normalize_spaces(self, text: str) -> str:
        special_spaces = {
            '{ ': '{',
            ' {': '{',
            '} ': '}',
            ' }': '}',
            '[ ': '[',
            ' [': '[',
            '] ': ']',
            ' ]': ']',
            '( ': '(',
            ' (': '(',
            ') ': ')',
            ' )': ')',
        }
        if not '  ' in text: return text
        while True:
            text = text.replace('  ', ' ')
            if not '  ' in text: break
        for special_space in special_spaces:
            text = text.replace(special_space, special_spaces[special_space])
        return text
    
    def unique_string_no_space(self, items: list) -> str:
        if type(items) is str:
            items = items.split()
        elif type(items) is not list:
            items = [items]
        text = ''
        for item in items:
            if not item:
                continue
            item = str(item)
            item = self.without_acute(item)
            text += item.lower().replace(' ','')
        return text
    
    def unique_string(self, items: str) -> str:
        text = ''
        for item in items:
            if not item:
                continue
            item = self.without_acute(item)
            item = self.normalize_spaces(item)
            text += item.lower() + ' '
        text = text[:-1]
        return text
    
    def normal_text(self, items: list) -> str:
        if type(items) is str:
            items = items.split()
        elif type(items) is not list:
            items = [items]
        text = ''
        for item in items:
            if not item:
                continue
            item = self.normalize_spaces(item)
            text += item + ' '
        text = text[:-1]
        text = self.normalize_spaces(text)
        return text

    def cambiar_estado(self, estado):
        ESTADOS = {
            'bajacalifornia': 'BCN',
            'bajacalifornianorte':"BCN",
            'bajacaliforniasur':"BCS",
            'aguascalientes':"AGS",
            'campeche':"CAM",
            'chihuahua':"CHI",
            'chiapas':"CHS",
            'coahuila':"COH",
            'colima':"COL",
            'distritofederal':"CDMX",
            'df':"CDMX",
            'ciudaddemexico':"CDMX",
            'cdmx':"CDMX",
            'durango':"DUR",
            'guerrero':"GRO",
            'guanajuato':"GTO",
            'hidalgo':"HGO",
            'jalisco':"JAL",
            'edo.demexico':"MEX",
            'estadodemexico':"MEX",
            'estadomexico':"MEX",
            'edomexico':"MEX",
            'mexico':"MEX",
            'michoacan':"MIC",
            'morelos':"MOR",
            'nayarit':"NAY",
            'nuevoleon':"NL",
            'oaxaca':"OAX",
            'puebla':"PUE",
            'quintanaroo':"QR",
            'queretaro':"QRO",
            'sinaloa':"SIN",
            'sanluispotosi':"SLP",
            'sonora':"SON",
            'tabasco':"TAB",
            'tamaulipas':"TAM",
            'tlaxcala':"TLX",
            'veracruz':"VER",
            'yucatan':"YUC",
            'zacatecas':"ZAC",
        }
        unique_edo = self.unique_string_no_space(estado.split())
        if unique_edo in ESTADOS:
            estado = ESTADOS[unique_edo]
        return estado




import os
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
import smtplib


class ErrorEmail:
    def __init__(self, message, email_settings, name_project = 'No hay nombre del projecto'):
        self.msg = MIMEMultipart()
        self.server = smtplib.SMTP_SSL(email_settings['smtp_server'])
        self.server.login(email_settings['user'], email_settings['password'])
        self.message = message
        self.sender = email_settings['sender']
        self.subject = f'ERROR EN {name_project}'
        self.receiver = email_settings['receiver']

    def send(self):
        self.msg['From'] = self.sender
        self.msg['Subject'] = self.subject
        self.msg['To'] = self.receiver
        self.msg.attach(MIMEText(self.message, 'plain'))
        self.server.sendmail(self.msg['From'], self.msg['To'], self.msg.as_string())
        self.server.quit()


class CatchErrors:
    
    def __init__(self, name_project = 'No hay nombre del projecto', email_settings = None):
        self.name_project = name_project
        self.email_settings = email_settings
        if not self.email_settings:
            self.email_available = False
        else:
            self.email_available = True

    def show_error(self, e: Exception, send_email: bool = False)->str:
        import os
        import datetime
        info_exc = os.sys.exc_info()
        exc_type, exc_obj, exc_tb = info_exc
        file = exc_tb.tb_frame.f_code.co_filename
        function_data = str(exc_tb.tb_frame).split()
        function_data = function_data[-1][:-1]
        now = datetime.datetime.now()
        now = now.strftime("%d/%m/%Y %H:%M:%S")
        
        error = f'ERROR INFO\nTipo: {exc_type}\nArchivo: {file}\nFuncion: {function_data}\nLinea: {exc_tb.tb_lineno}\nError: {e}\nFecha: {now}'
        
        if send_email:
            if not self.email_available:
                raise Exception('No hay configuración de email')
            email = ErrorEmail(error, self.email_settings, self.name_project)
            email.send()
        
        return error
